#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : lists.py
# Contents: auxiliary functions for list processing
# Author  : Christian Borgelt
# History : 2009.08.?? file created
#           2009.08.14 functions coins and xcoins programmed
#           2010.06.22 list functions made a separate file
#-----------------------------------------------------------------------

def integrate (list):
    '''Integrate a list of numbers.
list:   list to integrate
returns integral as a list'''
    integ = list[0:1]           # sum elements from start of list
    for i in xrange(1,len(list)):
        integ.append(integ[-1]+list[i])
    return integ                # iteratively expand sum with next elem.

#-----------------------------------------------------------------------

def coins (l1, l2, sorted=False):
    '''Find the number of equal entries (coincidences) in two lists.
l1, l2: lists to process
sorted: whether the lists are sorted
        (and thus need not be sorted first)
Note: If there are no duplicate entries, or duplicate entries are
to be disregarded, len(set(l1).intersection(l2)) is faster.'''
    if not sorted: l1.sort(); l2.sort()
    k1 = len(l1)-1; k2 = len(l2)-1; n = 0
    while k1 >= 0 and k2 >= 0:
        if   l1[k1] > l2[k2]:  k1 -= 1
        elif l1[k1] < l2[k2]:  k2 -= 1
        else: n += 1; k1 -= 1; k2 -= 1
    return n

#-----------------------------------------------------------------------

def xcoins (l1, l2, tolerance, sorted=False):
    '''Find the number of entry pairs in two lists
that differ by no more than 'tolerance'.
l1, l2:    lists to process
tolerance: maximum allowed difference between entries
sorted:    whether the lists are sorted
           (and thus need not be sorted first)'''
    if tolerance <= 0: return len(set(l1).intersection(l2))
    # if tolerance <= 0: return coins(l1, l2, sorted)
    if not sorted: l1.sort(); l2.sort()
    k1 = len(l1)-1; k2 = len(l2)-1; n = 0
    while k1 >= 0 and k2 >= 0:
        if   l1[k1] -tolerance > l2[k2]: k1 -= 1
        elif l1[k1] +tolerance < l2[k2]: k2 -= 1
        else:           n += 1; k1 -= 1; k2 -= 1
    return n

#-----------------------------------------------------------------------
# Main
#-----------------------------------------------------------------------

if __name__ == '__main__':
    a = [1,1,2,2,4,6,8,9]
    b = [1,2,2,3,5,7,8,9]
    print len(set(a).intersection(b))
    print coins(a,b)
    print xcoins(a,b,1)
